@@ -30,6 +30,7 @@ use Carp qw(croak);
 use Fcntl ();
 
 use AnyEvent ();
+use AnyEvent::Util ();
 
 our @ISA;
 
@@ -101,7 +102,7 @@ sub signal {
       or croak "AnyEvent->signal called with illegal cb argument '$arg{cb}'";
    delete $arg{cb};
  
-   eval "require POSIX; 0 < &POSIX::SIG$arg{signal}"
+   defined AnyEvent::Util::sig2num $arg{signal}
       or croak "AnyEvent->signal called with illegal signal name '$arg{signal}'";
    delete $arg{signal};
  
